#!/usr/bin/env python3

"""Dumps to standard output all information on a particular eddy,
specified by date and eddy_index, or by node index, or by shape index.

"""

import argparse
import shapefile
from os import path
import pprint
import sys
import util_eddies
import report_graph
import json

parser = argparse.ArgumentParser()
parser.add_argument("directory", help = "SHPC directory")
parser.add_argument("orientation", choices = ["Anticyclones", "Cyclones"])
group = parser.add_mutually_exclusive_group()
group.add_argument("-i", "--ishape", type = int)
group.add_argument("-n", "--node", type = int)
args = parser.parse_args()
SHPC = util_eddies.SHPC_class(args.directory, def_orient = args.orientation)

if args.ishape:
    ishape = args.ishape
else:
    if args.node:
        with open("e_overestim.txt") as f: e_overestim = int(f.read())
        date, eddy_index = report_graph.node_to_date_eddy(args.node,
                                                          e_overestim)
    else:
        reply = input("date, eddy_index = ? ").split(",")
        date = int(reply[0])
        eddy_index = int(reply[1])
        assert eddy_index >= 1

    ishape = SHPC.comp_ishape(date, eddy_index, 0, args.orientation)

print("ishape =", ishape)

for layer in ["extremum", "outermost_contour", "max_speed_contour", "centroid"]:
    print()

    try:
        shapeRec = SHPC.get_reader(0, layer, args.orientation)\
                       .shapeRecord(ishape)
    except AttributeError:
        pass
    else:
        print(layer, ":")

        if shapeRec.shape.shapeType == shapefile.NULL:
            dct = shapeRec.record.as_dict()
            pprint.pprint(dct)
            print("null shape")
        else:
            pprint.pprint(shapeRec.__geo_interface__)
