#!/usr/bin/env python3

"""Note: "ishape_last.txt" is only created with option renumber. If
this module is run as a script then module accept must be
provided. There is an example in Inst_eddies/Tests/Input.

"""

import shapefile
from os import path
import numpy as np

def filter(input_dir, output_dir, renumber, accept_func):
    with shapefile.Reader(path.join(input_dir, "extremum")) as extremum, \
         shapefile.Reader(path.join(input_dir, "outermost_contour")) \
         as outermost_cont, \
         shapefile.Reader(path.join(input_dir, "max_speed_contour")) \
         as max_speed_cont, \
         shapefile.Writer(path.join(output_dir, "extremum")) as extremum_filt, \
         shapefile.Writer(path.join(output_dir, "outermost_contour")) \
         as outermost_cont_filt, \
         shapefile.Writer(path.join(output_dir, "max_speed_contour")) \
         as max_speed_cont_filt:
        extremum_filt.fields = extremum.fields[1:]
        outermost_cont_filt.fields = outermost_cont.fields[1:]
        max_speed_cont_filt.fields = max_speed_cont.fields[1:]

        if renumber:
            last_date_written = None
            ishape_last = []

        for shape_rec_extr, shape_rec_outer, shape_rec_max \
            in zip(extremum, outermost_cont, max_speed_cont):
            if accept_func(shape_rec_extr, shape_rec_outer, shape_rec_max):
                if renumber:
                    if shape_rec_extr.record.date == last_date_written:
                        eddy_index += 1
                    else:
                        ishape_last.append((last_date_written,
                                            len(extremum_filt) - 1))
                        eddy_index = 1
                        last_date_written = shape_rec_extr.record.date

                for sr, w in zip([shape_rec_extr, shape_rec_outer,
                                  shape_rec_max],
                                 [extremum_filt, outermost_cont_filt,
                                  max_speed_cont_filt]):
                    r = sr.record
                    if renumber: r["eddy_index"] = eddy_index
                    w.record(*r)
                    w.shape(sr.shape)

        if renumber:
            ishape_last.append((last_date_written, len(extremum_filt) - 1))
            del ishape_last[0] # (None, - 1)
            assert len(ishape_last) != 0

            # Replace last date written by repetition of date factor:
            ishape_last = np.array(ishape_last)
            ishape_last[:- 1, 0] = np.ediff1d(ishape_last[:, 0])
            ishape_last[- 1, 0] = 1

            file = path.join(output_dir, "ishape_last.txt")

            with open(file, "w") as f:
                for repet_factor, i in ishape_last:
                    f.write(repet_factor * (str(i) + "\n"))

if __name__ == "__main__":
    import argparse
    import accept

    parser = argparse.ArgumentParser()
    parser.add_argument("input_dir",
                        help = "containing the three input shapefiles")
    parser.add_argument("output_dir",
                        help = "containing the three filtered shapefiles")
    parser.add_argument("--renumber", help = "reset indices of eddies",
                        action = "store_true")
    args = parser.parse_args()
    filter(args.input_dir, args.output_dir, args.renumber, accept.accept)
