program test_get_1_outerm

  ! Libraries:
  use jumble, only: csvread, deg_to_rad, rad_to_deg, pi
  use shapelib, only: shpfileobject, ftdouble, shpt_polygon, shpclose
  use shapelib_03, only: shp_create_03, dbf_add_field_03, &
       shp_append_object_03, dbf_write_attribute_03

  use config_m, only: config, min_radius
  use derived_types, only: eddy, shpc_slice_handler, null_ssh_contour, &
       missing_speed, ssh_contour
  use get_1_outerm_m, only: get_1_outerm
  use input_ssh_m, only: input_ssh
  use shpc_close_m, only: shpc_close
  use shpc_create_m, only: shpc_create
  use write_eddy_m, only: write_eddy

  implicit none

  integer:: date = 0, i, ifield, ishape

  real, allocatable:: ssh(:, :)
  ! (1 - max_radius(1):nlon + max_radius(1), nlat) if the grid is periodic
  ! in longitude, else (nlon, nlat). Sea-surface height, in m.

  real, allocatable:: u(:, :), v(:, :)
  ! (1 - max_radius(1):nlon + max_radius(1), nlat) if periodic, else
  ! (nlon, nlat) wind, in m s-1

  integer:: ind_targ_extr(2) = [19, 11]
  ! indices in the two dimensions of target extremum

  real:: innermost_level = 0.2933
  ! level of innermost contour, for each extremum

  logical:: cyclone = .true.

  real corner(2)
  ! longitude and latitude of the corner of the whole grid, in rad

  real step(2) ! longitude and latitude steps, in rad

  integer nlon, nlat
  ! size of ssh array in input NetCDF, assuming no repeated point if
  ! the grid is global

  logical periodic ! grid is periodic in longitude
  TYPE(shpc_slice_handler) hshp
  type(eddy) e

  real, allocatable:: outside_points(:, :) ! (2, :) longitude and
  ! latitude, in rad, of all the significant extrema, except the
  ! target extremum

  logical:: grid_lon_lat = .true.
  integer, parameter:: n_max_cont = 31

  type(ssh_contour) cont_list(n_max_cont + 1)
  ! Contour list. The outermost contour is element n_cont. The
  ! contours are in monotonic order of ssh.

  integer n_cont ! number of good contours found and stored in cont_list
  TYPE(shpfileobject) hshp_cont_list

  namelist /main_nml/ ind_targ_extr, innermost_level, cyclone, date, &
       grid_lon_lat

  !----------------------------------------------------------------

  write(unit = *, nml = main_nml)
  print  *, "test_get_1_outerm: Enter namelist main_nml."
  read(unit = *, nml = main_nml)
  write(unit = *, nml = main_nml)
  call config
  call input_ssh(corner, step, nlon, nlat, periodic, ssh, u, v)
  call csvread("outside_points.csv", outside_points, first_r = 3)
  outside_points = transpose(outside_points) * deg_to_rad
  e%extr%coord_proj = ind_targ_extr
  e%extr%coord = corner + (ind_targ_extr - 1) * step
  call get_1_outerm(e%out_cont, cont_list, n_cont, cyclone, e%extr%coord, &
       innermost_level, outside_points, ssh, corner, step, &
       min_area = pi * (min_radius * 1e3)**2)

  if (e%out_cont%closed) then
     e%extr%ssh = ssh(ind_targ_extr(1), ind_targ_extr(2))
     e%cyclone = cyclone
     e%speed_cont = null_ssh_contour()
     e%max_speed = missing_speed
     e%radius4 = - 1
     call shpc_create(hshp, shpc_dir = "SHPC", cyclone = cyclone, slice = 0, &
          grid_lon_lat = grid_lon_lat)
     call write_eddy(e, hshp, date, i = 1)
     write(hshp%unit, fmt = *) 0
     CALL shpc_close(hshp)
     print *, 'test_get_1_outerm: Created "SHPC".'

     ! cont_list shapefile:

     call shp_create_03("cont_list", shpt_polygon, hshp_cont_list)
     call dbf_add_field_03(ifield, hshp_cont_list, 'ssh', ftdouble, &
          nwidth = 13, ndecimals = 6)

     do i = 1, n_cont
        call shp_append_object_03(ishape, hshp_cont_list, shpt_polygon, &
             cont_list(i)%points * rad_to_deg)
        call dbf_write_attribute_03(hshp_cont_list, ishape, ifield, &
             cont_list(i)%ssh)
     end do

     CALL shpclose(hshp_cont_list)
     print *, 'test_get_1_outerm: Created shapefile "cont_list".'
  else
     print *, "test_get_1_outerm: Could not find an outermost contour."
  end if

end program test_get_1_outerm
