program test_good_contour

  ! Libraries:
  use contour_531, only: polyline
  use jumble, only: new_unit, count_lines
  use netcdf, only: nf90_nowrite
  use netcdf95, only: nf95_open, nf95_inquire_dimension, nf95_get_var, &
       nf95_inq_varid, nf95_close, nf95_inquire_variable
  use jumble, only: assert
  use shapelib, only: shpt_polygon, shpfileobject, ftdouble, shpclose
  use shapelib_03, only: shp_create_03, dbf_add_field_03, &
       shp_append_object_03, dbf_write_attribute_03
  
  use good_contour_m, only: good_contour

  implicit none

  character(len = :), allocatable:: filename
  integer nx, ny, n_out, i
  type(polyline) c
  real:: level = 0.
  real, allocatable:: Z(:, :)
  real:: inside_point(2) = [0., 0.]
  real, allocatable:: outside_points(:, :) ! (2, n_out)
  real:: step(2) = [1., 1.]
  real:: corner(2) = [1., 1.]
  TYPE(shpfileobject) shphandle
  integer field_number, shape_number
  integer length, status, ncid, varid, unit
  integer, allocatable:: dimids(:)
  character(len = 10):: varname = "z"

  namelist /main_nml/ inside_point, level, step, corner, varname

  !-------------------------------------------------------------------

  call get_command_argument(number = 1, length = length, status = status)
  call assert(status == 0, "Required argument: netCDF-file")
  allocate(character(len = length):: filename)
  call get_command_argument(1, filename)

  write(unit = *, nml = main_nml)
  print *, "Enter namelist main_nml."
  read(unit = *, nml = main_nml)
  write(unit = *, nml = main_nml)

  print *, "Reading from ", filename, "..."
  call nf95_open(filename, nf90_nowrite, ncid)
  call nf95_inq_varid(ncid, varname, varid)
  ! Assume ndims >= 2 and take the first 2D slice:
  call nf95_inquire_variable(ncid, varid, dimids = dimids)
  call nf95_inquire_dimension(ncid, dimids(1), nclen = nx)
  call nf95_inquire_dimension(ncid, dimids(2), nclen = ny)
  allocate(z(nx, ny))
  call nf95_get_var(ncid, varid, z)

  call nf95_close(ncid)

  print *, 'Reading from "outside_points.csv"...'
  call new_unit(unit)
  open(unit, file = "outside_points.csv", status = "old", action = "read", &
       position = "rewind")
  call count_lines(unit, n_out)
  rewind(unit)
  allocate(outside_points(2, n_out))
  if (n_out /= 0) read (unit, fmt = *) outside_points
  close(unit)
  print *, "outside_points:"

  do i = 1, n_out
     print *, outside_points(:, i)
  end do

  c = good_contour(corner, step, z, level, inside_point, outside_points)

  if (c%n_points /= 0) then
     call shp_create_03("test_good_contour", shpt_polygon, shphandle)
     call dbf_add_field_03(field_number, shphandle, 'level', ftdouble, &
          nwidth = 13, ndecimals = 6)
     call shp_append_object_03(shape_number, shphandle, shpt_polygon, c%points)
     call dbf_write_attribute_03(shphandle, shape_number, ifield = 0, &
          fieldvalue = level)
     CALL shpclose(shphandle)
     print *, "Created shapefile."
  else
     print *, "No good contour found."
  end if

end program test_good_contour
