program test_max_speed_contour_ssh

  use, intrinsic:: ieee_arithmetic, only: ieee_value, IEEE_QUIET_NAN
  use, intrinsic:: ISO_FORTRAN_ENV

  ! Libraries:
  use jumble, only: get_command_arg_dyn
  use netcdf, only: nf90_nowrite
  use netcdf95, only: nf95_open, nf95_close, nf95_inq_varid, nf95_gw_var
  use jumble, only: assert

  use get_var_m, only: get_var
  use max_speed_contour_ssh_m, only: max_speed_contour_ssh

  implicit none

  integer nlon, nlat

  character(len = :), allocatable:: adt_file, velocity_file
  integer ncid, varid
  real, allocatable:: ssh(:, :, :) ! (nlon, nlat, 1) sea-surface height, in m
  real, allocatable:: u(:, :), v(:, :) ! (nlon, nlat) wind, in m s-1

  integer:: ind_extr(2) = [19, 11]
  ! indices in the two dimensions of the extremum
  
  integer:: radius = 3

  namelist /main_nml/ ind_extr, radius

  !----------------------------------------------------------------

  call assert(COMMAND_ARGUMENT_COUNT() == 2, &
       "Required arguments: ADT-file velocity-file")
  call get_command_arg_dyn(1, adt_file)
  call get_command_arg_dyn(2, velocity_file)

  write(unit = error_unit, nml = main_nml)
  write(unit = error_unit, fmt = *) "Enter namelist main_nml."
  read(unit = *, nml = main_nml)
  write(unit = *, nml = main_nml)
  

  print *, "Reading from ", adt_file, "..."
  call nf95_open(adt_file, nf90_nowrite, ncid)
  call nf95_inq_varid(ncid, "adt", varid)
  call nf95_gw_var(ncid, varid, ssh)
  call nf95_close(ncid)

  nlon = size(ssh, 1)
  nlat = size(ssh, 2)
  allocate(u(nlon, nlat), v(nlon, nlat))

  print *, "Reading from ", velocity_file, "..."
  call nf95_open(velocity_file, nf90_nowrite, ncid)
  call get_var(periodic = .false., max_rad_lon = 0, values = u, ncid = ncid, &
       nlon = nlon, name = "ugos", &
       new_fill_value = ieee_value(0., IEEE_QUIET_NAN))
  call get_var(periodic = .false., max_rad_lon = 0, values = v, ncid = ncid, &
       nlon = nlon, name = "vgos", &
       new_fill_value = ieee_value(0., IEEE_QUIET_NAN))
  call nf95_close(ncid)

  print *, "level = ", max_speed_contour_ssh(ssh(:, :, 1), u, v, ind_extr, &
       radius)

end program test_max_speed_contour_ssh
