program test_set_max_speed

  ! Libraries:
  use jumble, only: new_unit, count_lines, get_command_arg_dyn, deg_to_rad

  use config_m, only: config
  use derived_types, only: eddy, shpc_slice_handler
  use input_ssh_m, only: input_ssh
  use read_eddy_m, only: read_eddy
  use set_max_speed_m, only: set_max_speed
  use shpc_close_m, only: shpc_close
  use shpc_create_m, only: shpc_create
  use shpc_open_m, only: shpc_open
  use write_eddy_m, only: write_eddy

  implicit none

  logical cyclone
  logical periodic ! grid is periodic in longitude
  integer nlon, nlat, unit, n, l, d, i
  real, allocatable:: ssh(:, :) ! (nlon, nlat) sea-surface height, in m
  real, allocatable:: u(:, :), v(:, :) ! (nlon, nlat) wind, in m s-1
  type(eddy) e
  real corner(2), step(2) ! in rad
  TYPE(shpc_slice_handler) hshp
  character(len = :), allocatable:: shpc_dir

  real, allocatable:: outside_points(:, :) ! (2, :) longitude and
  ! latitude, in rad, of all the extrema except the target extremum

  !----------------------------------------------------------------

  print *, "test_set_max_speed: cyclone = ? "
  read *, cyclone
  call config
  call input_ssh(corner, step, nlon, nlat, periodic, ssh, u, v)
  print *, "test_set_max_speed: Reading from shapefiles..."
  call get_command_arg_dyn(3, shpc_dir, "Required 3rd argument: SHPC-directory")
  call shpc_open(hshp, shpc_dir, cyclone, slice = 0, pszaccess = "rb")
  call read_eddy(e, d, i, hshp, ishape = 0)
  CALL shpc_close(hshp)

  print *, "test_set_max_speed: Reading from outside_points.csv..."
  call new_unit(unit)
  open(unit, file = "outside_points.csv", status = "old", action = "read", &
       position = "rewind")
  call count_lines(unit, n)
  n = n - 2 ! 2 title lines
  allocate(outside_points(2, n))
  rewind(unit)
  read(unit, fmt = *)
  read(unit, fmt = *)

  do l = 1, n
     read(unit, fmt = *) outside_points(:, l)
  end do
  
  outside_points = outside_points * deg_to_rad
  close(unit)
  step = 0.25  * deg_to_rad
  call set_max_speed(e, nint((e%extr%coord - corner) / step) + 1, &
       outside_points, ssh, u, v, corner, step)
  call shpc_create(hshp, shpc_dir = "SHPC", cyclone = cyclone, slice = 0, &
       grid_lon_lat = .true.)
  call write_eddy(e, hshp, d, i)
  CALL shpc_close(hshp)
  print *, 'test_set_max_speed: Created "SHPC".'

end program test_set_max_speed
