#!/usr/bin/env python3

"""We are assuming that all input NetCDF files are in the same
directory and that the time coordinate is in days since 1950-1-1."""

import argparse
import datetime
from os import path
import sys
from dateutil import parser
import netCDF4
import inst_eddies
import glob

def filename_generator(first_file, first_date, last_date, template):
    dirname = path.dirname(first_file)
    nc_file = first_file
    my_date = first_date
    
    while True:
        yield nc_file
        my_date += datetime.timedelta(1)
        if my_date > last_date: break
        basename = my_date.strftime(template)
        nc_file = path.join(dirname, basename)

        # The template may contain a wild card as long as there is no
        # more than one file matching it. A missing file is ok and
        # will be dealt with in inst_eddies.py.

        path_list = glob.glob(nc_file)

        if len(path_list) == 1:
            nc_file = path_list[0]
        elif len(path_list) >= 2:
            sys.exit("inst_eddies_Aviso.py:filename_generator: more than one "
                     "file matching template")

argparser = argparse.ArgumentParser()
argparser.add_argument("first_file",
                       help = "NetCDF file containing SSH and velocity at a "
                       "single date")
argparser.add_argument("slice", type = int,
                       help = "slice number to create")
argparser.add_argument("--template",
                       help = "template of basename with date.strftime format "
                       "codes (and possible shell wildcards), only used for "
                       "several dates")
argparser.add_argument("-l", "--last_date", help = "%%Y-%%m-%%d")
argparser.add_argument("-b", "--bbox", nargs=4, type = float,
                       metavar = ("xmin", "xmax", "ymin", "ymax"))
args = argparser.parse_args()

# Get the first date from the time coordinate in the NetCDF file. (In
# general, we cannot get it from the name of the first file because
# the first file may be the only file, and then there is no template
# option and the file name might not contain the date.)
with netCDF4.Dataset(args.first_file) as f:
    d = f["time"][:].item()

d = round(d)
first_date = datetime.date(1950, 1, 1) + datetime.timedelta(d)
#--

if args.last_date:
    last_date = parser.parse(args.last_date).date()
    
    if last_date > first_date and not args.template:
        sys.exit("template option required for several dates")
else:
    last_date = first_date

# We choose to use d as the first date index. We could choose anything.
inst_eddies.loop_inst_eddies(filename_generator(args.first_file, first_date,
                                                last_date, args.template),
                             args.bbox, d, my_slice = args.slice)
